function [hawvfm,haspgrm,haclrbr,hapsd]=Plot_SPGRM(DATA,File,ChTbl,PSD,SPGRM,ChID,Xunit,DataXLIM,DataYLIM,FreqLIM,CLIM,PsdYLIM,PsdXScale,UsgsNM,PlotSkip,MvAvg)
% Plot Waveform, Spectrogram and PSD
%
% Usage : [hawvfm,haspgrm,haclrbr,hapsd] = Plot_SPGRM(DATA,File,ChTbl,PSD,SPGRM,Xunit,DataXLIM,DataYLIM,CLIM,PsdYLIM,PsdXScale,UsgsNM,PlotSkip,MvAvg);
%
% *** INPUT Parameters ***
% DATA,File,ChTbl,PSD,SPGRM : See ReadWIN.m
% ChID      : Channel ID to be plotted (char). Ex) '00A0' or [] (Empty means the lowest ChID is selected) 
% Xunit     : Unit of Xaxis. 'Sec' or 'Minute' or 'Hour' or 'Clock'. Default is 'Sec'.
% DataXLIM  : Xaxis limits (time) of the waveform plot. [Xmin Xmax] or [] (Display whole range). If Xunit is 'Clock', DataXLIM must be date datevec format like {[yyyy mm dd HH MM SS],[yyyy mm dd HH MM SS]}.
% DataYLIM  : Yaxis limits (time) of the waveform plot. [Ymin Ymax] or [] (Display whole range)
% FreqLIM   : Xaxis limits (frequency) of the spectrogram and PSD plot. [Fmin Fmax] or [] (Display whole range).
% CLIM      : Color limits of the  spectrogram. [Cmin Cmax] or [] (Display whole range)
% PsdYLIM   : Yaxis limits of the PSD plot. [Ymin Ymax] or [] (Display whole range)
% PsdXScale : Xaxis scale of the  PSD plot. 'log' or 'linear' or '' or []. Default is 'linear'.
% UsgsNM    : Plot USGS noise model(NHNM/NLNM) on the PSD axes. 'Yes' or 'No' or ''. Default is 'Yes'
% PlotSkip  : Number of skipping data points for waveform plot. Default is 1 (no skip).
% MvAvg     : Moving average for smoothing the PSD plot. Default is 1 (do not apply moving average).
%
% *** UPDATES ***
% 2008/July/18(Fri) : Original by Koji MIYAKAWA
%

clf;
% ---------------------------
% Parameter Initialization
%
if exist('ChID','var') && ~isempty(ChID)                % Check ChID
        TarCh = find(hex2dec(ChTbl.ChID)==hex2dec(ChID));
    if isempty(TarCh)
        error([ChID,' is not found in the channel table.\n']);
    end
else
    TarCh = 1;
end
TarCol = File.ColChs(TarCh);
TarRow = File.RowChs(TarCh);
if exist('Xunit','var') && strcmpi(Xunit,'Minute')      % Check Xunit
    TIME = (0:size(DATA{TarCol},1)-1)'/File.SmpFrq(TarCol)/60;  % Minute
    SPGRM{TarCol}(TarRow).T = SPGRM{TarCol}(TarRow).T/60;
    Xlabel    = '/Min';
elseif exist('Xunit','var') && strcmpi(Xunit,'Hour')    % Hour
    TIME = (0:size(DATA{TarCol},1)-1)'/File.SmpFrq(TarCol)/3600;
    SPGRM{TarCol}(TarRow).T = SPGRM{TarCol}(TarRow).T/3600;
    Xlabel    = '/Hour';
elseif exist('Xunit','var') && strcmpi(Xunit,'Clock')   % Clock
    TIME = (0:size(DATA{TarCol},1)-1)'/File.SmpFrq(TarCol)/86400+File.StNum;
    SPGRM{TarCol}(TarRow).T = SPGRM{TarCol}(TarRow).T/86400+File.StNum;
    if exist('DataXLIM','var') && ~isempty(DataXLIM)
        DataXLIM  = [datenum(DataXLIM{1}) datenum(DataXLIM{2})];
    end
    Xlabel    = '';
else                                                    % Sec
    TIME = (0:size(DATA{TarCol},1)-1)'/File.SmpFrq(TarCol);
    Xunit     = 'Sec';
    Xlabel    = '/Sec';
end
if ~exist('DataXLIM','var') || isempty(DataXLIM)        % Check DataXLIM
    DataXLIM = [TIME(1) TIME(end)+TIME(2)-TIME(1)];
end
if ~exist('FreqLIM','var') || isempty(FreqLIM)          % Check FreqLIM
    FreqLIM = [PSD{TarCol}.F(1) PSD{TarCol}.F(end)];
end
if ~exist('PsdYLIM','var') || isempty(PsdYLIM)          % Check PsdYLIM
    PsdYLIM = [min(PSD{TarCol}.PSD(5:end,TarRow)) max(PSD{TarCol}.PSD(5:end,TarRow))]; % Omit PSD.PSD(1:4,TarCh) to remove the effect of its DC offset
end
if ~exist('PlotSkip','var') || isempty(PlotSkip)        % Check PlotSkip
    PlotSkip = 1;
end
if exist('MvAvg','var') && ~isempty(MvAvg) && MvAvg > 1 % Check MvAvg
    if ~rem(MvAvg,2)
        error('MvAvg Error : Number of data points for moving average must be odd number.');
    else
        a = 1;
        b = ones(1,MvAvg)/MvAvg;
        PSD{TarCol}.PSD(:,TarRow) = filtfilt(b,a,PSD{TarCol}.PSD(:,TarRow));
    end
else
    MvAvg = 1;
end
if exist('File','var') && isfield(File,'BPFfc') && ~isempty(File.BPFfc) % Check File.BPFfc
    if isnan(File.BPFfc(1))
        FiltStr = ['LPF=',num2str(File.BPFfc(2)),'Hz'];
    elseif isnan(File.BPFfc(2))
        FiltStr = ['HPF=',num2str(File.BPFfc(1)),'Hz'];
    else
        FiltStr = ['BPF=',num2str(File.BPFfc(1)),'-',num2str(File.BPFfc(2)),'Hz'];
    end
else
    FiltStr = 'No Filter';
end
if isempty(File.SPGRM_WndwSec)                          % Check File.SPGRM_WndwSec
    SPGRM_WndwSec = 1/(SPGRM{TarCol}(TarRow).F(2)-SPGRM{TarCol}(TarRow).F(1));
else
    SPGRM_WndwSec = File.SPGRM_WndwSec;
end
if isempty(File.SPGRM_OvlpPct)                          % Check File.SPGRM_OvlpPct
    SPGRM_OvlpPct = 50;
else
    SPGRM_OvlpPct = File.SPGRM_OvlpPct;
end
if isempty(File.PSD_WndwSec)                            % Check File.PSD_WndwSec
    PSD_WndwSec = 1/(PSD{TarCol}.F(2)-PSD{TarCol}.F(1));
else
    PSD_WndwSec = File.PSD_WndwSec;
end
if isempty(File.PSD_OvlpPct)                            % Check File.PSD_OvlpPct
    PSD_OvlpPct = 50;                                   % Empty variable means default(50%)
else
    PSD_OvlpPct = File.PSD_OvlpPct;
end

% ---------------------------
% Plot Waveform
%
YMEAN  = mean(DATA{TarCol}(1:PlotSkip:end,TarRow));
hawvfm = subplot(4,1,1); plot(TIME(1:PlotSkip:end),DATA{TarCol}(1:PlotSkip:end,TarRow)-YMEAN,'Tag','WV'); grid on;
htwvfm = title(sprintf('Waveform and Spectrogram (DCoffset=%G%s, %s, PlotSkip=%dpts)',YMEAN,ChTbl.Unit{TarCh},FiltStr,PlotSkip));
ylabel(ChTbl.Unit{TarCh});
xlim(DataXLIM);
if strcmpi(Xunit,'Clock')
    datetick(hawvfm,'x','HH:MM','keeplimits');
else
    DoubleTick(hawvfm,'x');     % Double the number of the XTick
end
if exist('DataYLIM','var') && ~isempty(DataYLIM)
    ylim(DataYLIM);
end
DoubleTick(hawvfm,'y');     % Double the number of the YTick

% ---------------------------
% Plot Spectrogram
%
haspgrm = subplot(4,1,[2 3]); imagesc(SPGRM{TarCol}(TarRow).T,SPGRM{TarCol}(TarRow).F,10*log10(SPGRM{TarCol}(TarRow).PSD),'Tag','SPGRM'); axis('xy');
% colormap(gray);                                           % Gray Scaale
xlabel(sprintf('Time%s (Window=%Gsec, Overlap=%G%%)',Xlabel,SPGRM_WndwSec,SPGRM_OvlpPct));
ylabel('Frequency/Hz');
xlim(DataXLIM);
if strcmpi(Xunit,'Clock')
    datetick(haspgrm,'x','HH:MM','keeplimits');
else
    DoubleTick(haspgrm,'x');     % Double the number of the XTick
end
ylim(FreqLIM);
DoubleTick(haspgrm,'y');    % Double the number of the YTick
if exist('CLIM','var') && ~isempty(CLIM)
    caxis(CLIM);
end
haclrbr = colorbar('SouthOutside');                         % Colorbar
xlabel(haclrbr,sprintf('PSD/(%s)^2/Hz in dB',ChTbl.Unit{TarCh}));

% ---------------------------
% Plot PSD
%
hapsd = subplot(4,1,4); semilogy(PSD{TarCol}.F,PSD{TarCol}.PSD(:,TarRow),'Tag','PSD'); grid on
htpsd = title('PSD using whole waveform data');
xlabel(sprintf('Frequency/Hz (Window=%Gsec, Overlap=%G%%, MovingAvg=%dpts)',PSD_WndwSec,PSD_OvlpPct,MvAvg));
ylabel(sprintf('PSD/(%s)^2/Hz',ChTbl.Unit{TarCh}));
if exist('PsdXScale','var') && strcmpi(PsdXScale,'log')     % Set Xscale 'log'
    if exist('UsgsNM','var') && ~strcmpi(UsgsNM,'No')
        if strcmpi(ChTbl.Unit{TarCh},'m/s/s') || strcmpi(ChTbl.Unit{TarCh},'Differential of m/s')
            NMch = 4;
        elseif strcmpi(ChTbl.Unit{TarCh},'m/s') || strcmpi(ChTbl.Unit{TarCh},'Integral of m/s/s') || strcmpi(ChTbl.Unit{TarCh},'Differential of m')
            NMch = 3;
        elseif strcmpi(ChTbl.Unit{TarCh},'m') || strcmpi(ChTbl.Unit{TarCh},'Integral of m/s')
            NMch = 2;
        else
            NMch = [];
        end
        if ~isempty(NMch)
            load('USGS_NLNM_NHNM.mat');
            hold on;
            semilogy(NHNM(:,1),NHNM(:,NMch),'r--',NLNM(:,1),NLNM(:,NMch),'k--');
            hold off;
            legend([strrep(ChTbl.StnNm{TarCh},'_','\_'),'-',ChTbl.Comp{TarCh}],'USGS-NHNM','USGS-NLNM');
        end
    end
    if FreqLIM(1) == 0
        if FreqLIM(2) < 0.01
            FreqLIM(1) = 10^floor(log10(FreqLIM(2)));
        else
            FreqLIM(1) = 0.01;     % Avoid log10(0)
        end
    end
    xlim(FreqLIM);
    set(hapsd,'XScale','log','XTick',10.^(fix(log10(FreqLIM(1))):floor(log10(FreqLIM(2)))));   % Set XTick to every figure in log scale
else
    xlim(FreqLIM);
    DoubleTick(hapsd,'x');      % Double the number of the XTick
end
ylim(PsdYLIM);
set(hapsd,'YTick',10.^(fix(log10(PsdYLIM(1))):floor(log10(PsdYLIM(2)))));   % Set YTick to every figure in log scale

% ---------------------------
% Text Information
%
CurAxes = gca;
axes('Position',[0 0 1 1],'Visible','off');
if File.RecSec < 3*3600                         % Set 'minute' if the data length is less than 3 hours
    DUR = [num2str(round(File.RecSec/60*10)/10),'min'];
else
    DUR = [num2str(round(File.RecSec/3600*10)/10),'hr'];
end
TITLE = sprintf('%s-%s (%s) %s %s - %s (%s)',strrep(ChTbl.StnNm{TarCh},'_','\_'),ChTbl.Comp{TarCh},ChTbl.ChID{TarCh},File.StDate,File.StTime,File.EdTime,DUR);
text(0.5,0.998,TITLE,'HorizontalAlignment','center','VerticalAlignment','top','FontSize',12,'FontWeight','bold');
set(gcf,'CurrentAxes',CurAxes);

% ---------------------------
% Position Control
%
Pawvfm     = get(hawvfm,'Position');            % Adjust Spectrogram Position
Paspgrm    = get(haspgrm,'Position');           % 
Paspgrm(4) = Pawvfm(2)-Paspgrm(2)-0.02;         % Extend Yaxis of the Spectrogram Window
set(haspgrm,'Position',Paspgrm);

Paclrbr    = get(haclrbr,'Position');           % Adjust Colorbar Position
Paclrbr(4) = Paclrbr(4)/3;                      % Shorten Yaxis of the Colorbar
set(haclrbr,'Position',Paclrbr,'XAxisLocation','bottom');

Papsd      = get(hapsd,'Position');             % Adjust PSD Position
Papsd(4)   = Paclrbr(2)-Papsd(2)-0.06;          % Extend Yaxis of the PSD Window
set(hapsd,'Position',Papsd);

set(htwvfm,'Units','normalized');               % Adjust Waveform Title Position
Ptwvfm     = get(htwvfm,'Position');
Ptwvfm(2)  = Ptwvfm(2)-0.04;
set(htwvfm,'Position',Ptwvfm);

set(htpsd,'Units','normalized');                % Adjust PSD Title Position
Ptpsd      = get(htpsd,'Position');
Ptpsd(2)   = Ptpsd(2)-0.04;
set(htpsd,'Position',Ptpsd);

pause(1);                                       % Adjust Wavefrom Position
P0adef     = get(0,'DefaultAxesPosition');
Pawvfm     = get(hawvfm,'Position');
if Pawvfm(1) ~= P0adef(1)
    set(hawvfm,'Position',[P0adef(1) Pawvfm(2) P0adef(3) Pawvfm(4)]);
end

% ===========================
function DoubleTick(ha,Axis)
% Double the number of the Tick
%
if strcmpi(Axis,'x')
    LIMstr  = 'XLim';
    TICKstr = 'XTick';
elseif strcmpi(Axis,'y')
    LIMstr  = 'YLim';
    TICKstr = 'YTick';
else
    return
end
LIM   = get(ha,LIMstr);
TICK  = get(ha,TICKstr);
dTICK = (TICK(2)-TICK(1))/2;
if LIM(2) > TICK(end)+dTICK
    TICK(end) = TICK(end)+dTICK;
end
set(ha,TICKstr',TICK(1):dTICK:TICK(end));
